String.prototype.format = function (e) { var t = this; if (arguments.length > 0) if (arguments.length == 1 && typeof e == "object") { for (var n in e) if (e[n] != undefined) { var r = new RegExp("({" + n + "})", "g"); t = t.replace(r, e[n]) } } else for (var i = 0; i < arguments.length; i++) if (arguments[i] != undefined) { var r = new RegExp("({)" + i + "(})", "g"); t = t.replace(r, arguments[i]) } return t };
var Helper ={
    getSpeedValue: function (speedUnit, speed) {
        var ret = 0;
        switch (speedUnit) {
            case "KT": ret = parseInt(speed * 1.852); break;
            case "KPH": ret = parseInt(speed); break;
            case "MPS": ret = parseInt(speed * 0.277777778); break;
            case "MPH": ret = parseInt(speed * 0.621371192); break;
            default: break;
        }
        return ret;
    },
    getSpeedUnit: function (speedUnit) {
        var ret = "";
        switch (speedUnit) {
            case "KT": ret = "kt"; break;
            case "KPH": ret = "km/h"; break;
            case "MPS": ret = "m/s"; break;
            case "MPH": ret = "mile/h"; break;
            default: break;
        }
        return ret;
    },
    getMileageValue: function (speedUnit, mileage) {
        var ret = 0;
        switch (speedUnit) {
            case "KT": ret = parseInt(mileage * 0.53995680345572); break;
            case "KPH": ret = parseInt(mileage); break;
            case "MPS": ret = parseInt(mileage * 1000); break;
            case "MPH": ret = parseInt(mileage * 0.62137119223733); break;
            default: break;
        }
        return ret;
    },
    getMileageUnit: function (speedUnit) {
        var ret = "";
        switch (speedUnit) {
            case "KT": ret = "mile"; break;
            case "KPH": ret = "km"; break;
            case "MPS": ret = "m"; break;
            case "MPH": ret = "mile"; break;
            default: break;
        }
        return ret;
    },
    getAddressByGeocoder: function(latlng,replyFunc){
        var url = "http://nominatim.openstreetmap.org/reverse?format=json&lat={0}&lon={1}&zoom=18&addressdetails=1".format(latlng.lat, latlng.lng);
        JSON.jsonp(url, function(result){ replyFunc(result.display_name);});
    },
    getLatLngByGeocoder: function(address,replyFunc){
        var url = "http://nominatim.openstreetmap.org/search?q={0}&format=json&polygon=1&addressdetails=1".format(address);
        JSON.jsonp(url, function(result){ 
            var res = new L.LatLng(result[0].lat, result[0].lon);
            replyFunc(res);
        });
    },
    createMap: function(option){
        var osm = L.tileLayer('http://{s}.tile.osm.org/{z}/{x}/{y}.png', { name: 'osm', attribution: '' });
        var map= L.map(option.target, { zoomControl: false, center: option.latLng, zoom: option.zoom }); 
        map.addLayer(osm);
        L.control.scale().addTo(map);
        L.control.zoom({ position: 'bottomright' }).addTo(map);
        return map;
    }
}

Helper.Protocol = {
    productFeatures: {
        None: 0,
        LatLng: 1,
        Alt: 2,
        Direct: 4,
        Speed: 8,
        Mileage: 16,
        GsmSignal: 32,
        GpsSignal: 64,
        Acc: 128,
        Static: 256,
        Voltage: 512,
        Battery: 1024,
        FuelSensor: 2048,
        TempSensor: 4096,
        OBD: 8192,
        RFIDCard: 16384,
        Holder: 32768,
        DrivingTime: 65536,
        Acc2: 131072
    },
    positionStatus: {
		"NONE": 0,
		"Acc": 1,
		"Static": 2,
		"Power": 4,
		"Charging": 8,
		"BeProtected": 16,
		"Acc2": 32
	},
	createPosInfo: function(ary) {
	    var posInfo = {};
	    posInfo.assetID = ary[0];
	    posInfo.imei = ary[1];
	    posInfo.protocolClass = ary[2];
	    posInfo.positionType = ary[3];
	    posInfo.dataType = ary[4];
	    if(ary[5] != null) {
	        posInfo.positionTime = moment(ary[5]).add('minutes', moment().zone() * -1);
	    }
	    else {
	        posInfo.positionTime = null;
	    }
	    if(ary[6] != null) {
	        posInfo.sysTime = moment(ary[6]).add('minutes', moment().zone() * -1);
	    }
	    else {
	        posInfo.sysTime = null;
	    }
	    if(ary[7] != null) {
	        posInfo.staticTime = moment(ary[7]).add('minutes', moment().zone() * -1);
	    }
	    else {
	        posInfo.staticTime = null;
	    }
	    posInfo.isRealTime = ary[8];
	    posInfo.isLocated = ary[9];
	    posInfo.satelliteSignal = ary[10];
	    posInfo.gsmSignal = ary[11];
	    posInfo.lat = ary[12];
	    posInfo.lng = ary[13];
	    posInfo.alt = ary[14];
	    posInfo.direct = ary[15];
	    posInfo.speed = ary[16];
	    posInfo.mileage = ary[17];
	    posInfo.launchHours = ary[18];
	    posInfo.alerts = ary[19];
	    posInfo.status = ary[20];
	    posInfo.originalAlerts = ary[21];
	    posInfo.originalStatus = ary[22];        
	    return posInfo;
	},
	createHisPosInfo: function(ary) {
	    var posInfo = {};
	    posInfo.positionTime = moment("1970-01-01")
	    .add('seconds', parseInt(ary[0]))
	    .add('minutes', moment().zone() * -1);
	    posInfo.lat = ary[1];
	    posInfo.lng = ary[2];
	    posInfo.direct = ary[3];
	    posInfo.speed = ary[4];
	    posInfo.timeSpan = ary[5];
	    posInfo.mileage = ary[6];
	    posInfo.alerts = ary[7];
	    posInfo.status = ary[8];
	    posInfo.eventType = ary[9];
	    return posInfo;
	},
	createAsset: function(ary){
		var index = 0;
		return {
				id: ary[index++],
				imei: ary[index++],
				name: ary[index++],
				tag: ary[index++],
				icon: "http://new.quiktrak.co/resources/images/asset/" + ary[index++],
				speedUnit: ary[index++],
				beMonitor: ary[index++],
				initMileage: ary[index++],
				initAccHours: ary[index++],
				state: ary[index++],
				prdtName: ary[index++],
				features: parseInt(ary[index++]),
				alerts: ary[index++],
				attr1: ary[index++],
				attr2: ary[index++],
				attr3: ary[index++],
				attr4: ary[index++],
				attr5: ary[index++]
		};
	},
    template: {
        trackInfo: '<li class="item-content"><div class="item-media"><i class="icon color-red {1}"></i></div><div class="item-inner"><div class="item-title" id="{0}"></div></div></li>',
        commandNode: '<li><a href="#" class="item-link item-content" onclick="{1}"><div class="item-inner"><div class="item-title-row"><div class="item-title">{0}</div></div></div></a></li>'
    },
	createTrackForm: function(asset, isPlayback){
		var ret = "";
        ret += Helper.Protocol.template.trackInfo.format('divPostime', 'ion-ios7-clock-outline');
        if(isPlayback)
        {
            ret +=  '<li class="item-content"> <div class="item-media"><i class="icon ion-ionic"></i></div> <div class="item-inner"> <div class="item-input"> <div class="range-slider"><input id="btnPlay" type="range" min="0" max="100" step="1"> </div> </div> </div> </li>';
        }
        if((Helper.Protocol.productFeatures.Acc & asset.features) > 0)
        {
        	ret += Helper.Protocol.template.trackInfo.format('divPosacc', 'ion-ios7-toggle-outline');
        }
        ret += Helper.Protocol.template.trackInfo.format('divPosspeed', 'ion-ios7-pulse');
        ret += Helper.Protocol.template.trackInfo.format('divPosmileage', 'ion-ios7-speedometer-outline');
        ret += Helper.Protocol.template.trackInfo.format('divPosaddress', 'ion-ios7-world-outline');
		//if(asset.features)
		return ret;
	},
	showTrackInfo: function(asset, posInfo, timeFormat){
		var latlng = [posInfo.lat, posInfo.lng];
		window.MarkerTrack.setLatLng(latlng);
		window.MapTrack.setView(latlng);
        if(posInfo.positionTime == null) {
            return;
        }
		Helper.getAddressByGeocoder(posInfo, function(addr){
			$$("#divPosaddress").html(addr);
		});
		if((Helper.Protocol.productFeatures.Acc & asset.features) > 0)
        {
        	if((Helper.Protocol.positionStatus.Acc & posInfo.status) > 0)
        	{
        		$$("#divPosacc").html("ACC&nbsp;ON");
        	}
        	else{
        		$$("#divPosacc").html("ACC&nbsp;OFF");
        	}
        }
		
		$$("#divPostime").html(posInfo.positionTime.format(timeFormat));
		$$("#divPosspeed").html(Helper.getSpeedValue(window.TargetAsset.speedUnit, posInfo.speed) + '&nbsp;' + Helper.getSpeedUnit(window.TargetAsset.speedUnit));
		$$("#divPosmileage").html(parseInt(Helper.getMileageValue(window.TargetAsset.speedUnit, posInfo.mileage)) + '&nbsp;' + Helper.getMileageUnit(window.TargetAsset.speedUnit));
	},
    showOBDInfo: function(obdInfo, timeFormat){
        $$("#divObdTime").html(moment(obdInfo.Time).add('minutes', moment().zone() * -1).format(timeFormat));

        if(obdInfo.PIDs["PID1F"]) {
            var ts = TimeSpan.FromSeconds(obdInfo.PIDs["PID1F"]);
            $$("#divObdPID1F").html("{0}:{1}:{2}".format(ts.hours(), ts.minutes(), ts.seconds()));
        }
        else{
            $$("#divPID1F").hide();
        }

        if(obdInfo.PIDs["PID88"]) {
            $$("#divObdPID88").html(obdInfo.PIDs["PID88"] + " L/H");
        }
        else {
            $$("#divPID88").hide();
        }
        if(obdInfo.PIDs["PID04"]) {
            $$("gagueCLV").show();
            window.OBDControls["CLV"].refresh(obdInfo.PIDs["PID04"]);
        }
        else {
            $$("#gagueCLV").hide();
        }
        if(obdInfo.PIDs["PID05"]) {
            $$("#gagueECT").show();
            window.OBDControls["ECT"].refresh(obdInfo.PIDs["PID05"]);
        }
        else {
            $$("#gagueECT").hide();
        }
        if(obdInfo.PIDs["PID0B"]) {
            $$("#gagueMAP").show();
            window.OBDControls["MAP"].refresh(obdInfo.PIDs["PID0B"]);
        }
        else {
            $$("#gagueMAP").hide();
        }
        if(obdInfo.PIDs["PID0C"]) {
            $$("#gagueRPM").show();
            window.OBDControls["RPM"].refresh(obdInfo.PIDs["PID0C"]);
        }
        else {
            $$("#gagueRPM").hide();
        }
        if(obdInfo.PIDs["PID0D"]) {
            $$("#gagueVSS").show();
            window.OBDControls["VSS"].refresh(obdInfo.PIDs["PID0D"]);
        }
        else {
            $$("#gagueVSS").hide();
        }
        if(obdInfo.PIDs["PID0E"]) {
            $$("#gagueEUT").show();
            window.OBDControls["EUT"].refresh(obdInfo.PIDs["PID0E"]);
        }
        else {
            $$("#gagueEUT").hide();
        }
        if(obdInfo.PIDs["PID0F"]) {
            $$("#gagueIAT").show();
            window.OBDControls["IAT"].refresh(obdInfo.PIDs["PID0F"]);
        }
        else {
            $$("#gagueIAT").hide();
        }
        if(obdInfo.PIDs["PID10"]) {
            $$("#gagueMAF").show();
            window.OBDControls["MAF"].refresh(obdInfo.PIDs["PID10"]);
        }
        else {
            $$("#gagueMAF").hide();
        }
        if(obdInfo.PIDs["PID11"]) {
            $$("#gagueTP").show();
            window.OBDControls["TP"].refresh(obdInfo.PIDs["PID11"]);
        }
        else {
            $$("#gagueTP").hide();
        }
        if(obdInfo.PIDs["PID42"]) {
            $$("#gaguePCM").show();
            window.OBDControls["PCM"].refresh(obdInfo.PIDs["PID42"]);
        }
        else {
            $$("#gaguePCM").hide();
        }
    },
	createButtons: function(asset){
		var ret = [];
        if((Helper.Protocol.productFeatures.LatLng & asset.features) > 0)
        {
            ret.push({
                text: window.LANGUAGE.ASSET_LIST_MSG01,
                bold: true,
                onClick: function () {
                    App.showPreloader();
                    trackAsset(function(){
                        $("#trackContent").html(Helper.Protocol.createTrackForm(window.TargetAsset));
                        mainView.router.load({pageName: 'track'});
                        thrStartTracking();
                    });
                }
            });
            ret.push({
                text: window.LANGUAGE.ASSET_LIST_MSG04,
                bold: true,
                onClick: function () {
                    App.showPreloader();
                    showPage("info", "asset.playBackRange", {}
                        , function(){
                            $$("#titleInfo").html(window.TargetAsset.name);
                            App.hidePreloader();
                        });
                }
            });
        }
        if((Helper.Protocol.productFeatures.OBD & asset.features) > 0)
        {
            ret.push({
                text: window.LANGUAGE.ASSET_LIST_MSG05,
                bold: true,
                onClick: function () {
                    App.confirm(window.LANGUAGE.COM_MSG05, function () {
                        App.showPreloader();
                        trackOBDInfo(
                            function(){
                                showPage("info", "asset.obdInfo", {} , function(){
                                    $$("#titleInfo").html(window.TargetAsset.name);
                                    window.OBDControls = {};
                                    window.OBDControls["VSS"] = new JustGage({ id: "gagueVSS", min: 0, max: 255, value: 0, title: "VSS", label: "km / h" });
                                    window.OBDControls["RPM"] = new JustGage({ id: "gagueRPM", min: 0, max: 16384, value: 0, title: "RPM", label: "r / min" });
                                    window.OBDControls["ECT"] = new JustGage({ id: "gagueECT", min: -40, max: 215, value: 0, title: "ECT", label: "℃" });
                                    window.OBDControls["EUT"] = new JustGage({ id: "gagueEUT", min: -64, max: 64, value: 0,  title: "EUT", label: "°" });
                                    window.OBDControls["MAP"] = new JustGage({ id: "gagueMAP", min: 0, max: 255, value: 0, title: "MAP", label: "kPa" });
                                    window.OBDControls["MAF"] = new JustGage({ id: "gagueMAF", min: 0, max: 655, value: 0, title: "MAF", label: "grams/sec" });
                                    window.OBDControls["IAT"] = new JustGage({ id: "gagueIAT", min: -40, max: 215, value: 0, title: "IAT", label: "℃" });
                                    window.OBDControls["TP"] = new JustGage({ id: "gagueTP", min: 0, max: 100, value: 0, title: "TP",  label: "%" });
                                    window.OBDControls["CLV"] = new JustGage({ id: "gagueCLV", min: 0, max: 100, value: 0, title: "CLV", label: "%" });
                                    window.OBDControls["PCM"] = new JustGage({ id: "gaguePCM", min: 0, max: 66, value: 0, title: "PCM", label: "V"});
                                    Helper.Protocol.showOBDInfo(window.TargetOBDInfo, getUserinfo().timeFormat);
                                    App.hidePreloader();
                                })
                            }
                        );
                    });
                }
            });
//          ret.push({
//              text: window.LANGUAGE.ASSET_LIST_MSG06,
//              bold: true,
//              onClick: function () {
//                  App.confirm(window.LANGUAGE.COM_MSG05, function () {
//                      App.showPreloader();
//                      JSON.request(URL_GET_OBD_CHECK.format(getUserinfo().pushCode, window.TargetAsset.id),
//                          function(result){
//                              if(result.IsOk){
//
//                              }
//                              else{
//
//                              }
//                              App.hidePreloader();
//                          },
//                          function(){ App.hidePreloader(); App.alert(window.LANGUAGE.COM_MSG01);  });
//                  });
//              }
//          });
        }
        return ret;
	}
};

Helper.Protocol["QT500"] = {
    createCommandPage: function(asset){

    }
};
